@@ -20,8 +20,8 @@
  * @author Sam Lang, Sam Meder
  * 
  * $RCSfile: globus_gsi_credential.c,v $
- * $Revision: 1.49 $
- * $Date: 2011/08/01 21:52:36 $
+ * $Revision: 1.49.6.1 $
+ * $Date: 2012/01/05 17:24:02 $
  */
 #endif
 
@@ -981,8 +981,7 @@ globus_gsi_cred_read_proxy_bio(
             sk_X509_push(certs, tmp_cert);
         }
         else if (strcmp(name, PEM_STRING_RSA) == 0 ||
-                 strcmp(name, PEM_STRING_DSA) == 0 ||
-                 strcmp(name, PEM_STRING_PKCS8INF) == 0)
+                 strcmp(name, PEM_STRING_DSA) == 0)
         {
             if (!PEM_get_EVP_CIPHER_INFO(header, &cipher))
             {
@@ -1014,6 +1013,33 @@ globus_gsi_cred_read_proxy_bio(
                 goto exit;
             }
         }
+        else if (strcmp(name, PEM_STRING_PKCS8INF) == 0)
+        {
+            PKCS8_PRIV_KEY_INFO *p8inf = NULL;
+
+            p8inf = d2i_PKCS8_PRIV_KEY_INFO(p8inf, &data, len);
+            if (p8inf == NULL)
+            {
+                GLOBUS_GSI_CRED_OPENSSL_ERROR_RESULT(
+                    result,
+                    GLOBUS_GSI_CRED_ERROR_READING_PROXY_CRED,
+                    (_GCRSL("Couldn't read pkcs8 key info from bio")));
+
+                goto exit;
+            }
+            handle->key = EVP_PKCS82PKEY(p8inf);
+
+            PKCS8_PRIV_KEY_INFO_free(p8inf);
+            if (handle->key == NULL)
+            {
+                GLOBUS_GSI_CRED_OPENSSL_ERROR_RESULT(
+                    result,
+                    GLOBUS_GSI_CRED_ERROR_READING_PROXY_CRED,
+                    (_GCRSL("Couldn't parse pkcs8 key")));
+
+                goto exit;
+            }
+        }
         else
         {
             GLOBUS_GSI_CRED_OPENSSL_ERROR_RESULT(
